/*
  ADAU1701_DSP - Library for calculating the filter coefficients and implementing the filters and crossovers
	for the Audiodeveloper DSP boards.
  Created by Neil Davis, December, 2020.
*/

#ifndef ADAU1701_DSP_H
#define ADAU1701_DSP_H

#include "Arduino.h"
#include <Shared_enums.h>    //this file defines the filter types--it must be included to use this library
#include <ADAU1701_I2C.h>
#include <Bilinear.h>

enum Xovers {none, BW_1, BW_3, LR_2, LR_4, LR_8, BW_2, BW_4, BW_5, BW_6, Bes_2, Bes_3, Bes_4, Bes_5, Bes_6,  LR_6, LR_10, LR_12, LR_14};    //This must agree with the list in HCI.h

class ADAU1701_DSP
{

  public:
    ADAU1701_DSP();

    Shared_enum::BQ_spec get_biquad_spec(float FC, Shared_enum::Xovers x_type);               //determine what filter types are needed and return that info in the biquad_spec structure
    Shared_enum::Xovers convert_Xover_code(byte freq_code);
    float convert_freq_code(bool TW_true_WS_false, byte freq_code);                           //use current HCI value to look u[ the corresponding floating point value
    void TW_Xover_calc(Shared_enum::BQ_spec BQ, word TW_BQ_addresses[5][4]);
    void WS_Xover_calc(Shared_enum::BQ_spec BQ, word WS_BQ_addresses[3][4]);
    void filter_calc(Shared_enum::Filter_spec Filter, word BQ_address);
    void phase_invert(bool passthrough, word polarity_addresses[2]); 

  private:

    Shared_enum::Xovers x_type;
    byte ADAU1701_coefs [5][4];   //five 28-bit values.  Will need to prepend 12 bits of zeroes to make into 40-bit data
    byte BQ_coefs [5][4];   //five 28-bit values.  Will need to prepend 12 bits of zeroes to make into 40-bit data
    float _coef_array[6];         //array to hold one set of biquad coefficents processed by Bilinear() and sent to coef_to_ADAU1701()

    void TW_biquad_load(byte biquad, float Freq, byte order, float q, word TW_BQ_addresses[5][4]);
    void WS_biquad_load(byte biquad, float Freq, byte order, float q, word WS_BQ_addresses[5][4]);
    void fill_TW_with_none(byte number_of_biquads, word TW_BQ_addresses[5][4]);
    void fill_WS_with_none(byte number_of_biquads, word WS_BQ_addresses[3][4]);
    void coef_passthrough();
    void coef_minus_1();
    void coef_to_ADAU1701(float coef_array[6]);
};

#endif
