/*  The Volume table converts the 100 volume values into logarithmically spaced values for the ADAU1701
     There is an Excel spreadsheet in the library directory that calculates these values.  PROGMEM is defined
     in <avr/pgmspace.h> -- it is a way of putting the data in progam (flash) memory instead of wasting RAM.
*/

const byte Main_Vol_table[] PROGMEM = {
  0x00, 0x03, 0x46, 0x00, 0x03, 0xAD, 0x00, 0x04, 0x20, 0x00, 0x04, 0xA0, 0x00, 0x05, 0x31, 0x00, 0x05, 0xD3,
  0x00, 0x06, 0x89, 0x00, 0x07, 0x55, 0x00, 0x08, 0x3B, 0x00, 0x09, 0x3C, 0x00, 0x0A, 0x5C, 0x00, 0x0B, 0xA0,
  0x00, 0x0D, 0x0B, 0x00, 0x0E, 0xA3, 0x00, 0x10, 0x6C, 0x00, 0x12, 0x6D, 0x00, 0x14, 0xAC, 0x00, 0x17, 0x32,
  0x00, 0x1A, 0x07, 0x00, 0x1D, 0x34, 0x00, 0x20, 0xC4, 0x00, 0x24, 0xC4, 0x00, 0x29, 0x40, 0x00, 0x2E, 0x49,
  0x00, 0x33, 0xEF, 0x00, 0x3A, 0x45, 0x00, 0x41, 0x61, 0x00, 0x49, 0x5B, 0x00, 0x52, 0x4F, 0x00, 0x5C, 0x5A,
  0x00, 0x67, 0x9F, 0x00, 0x74, 0x43, 0x00, 0x82, 0x73, 0x00, 0x92, 0x5E, 0x00, 0xA4, 0x3A, 0x00, 0xB8, 0x44,
  0x00, 0xCE, 0xC0, 0x00, 0xE7, 0xFA, 0x01, 0x04, 0x49, 0x01, 0x24, 0x0B, 0x01, 0x47, 0xAE, 0x01, 0x6F, 0xA9,
  0x01, 0x9C, 0x86, 0x01, 0xCE, 0xDC, 0x02, 0x07, 0x56, 0x02, 0x46, 0xB4, 0x02, 0x8D, 0xCE, 0x02, 0xDD, 0x95,
  0x03, 0x37, 0x18, 0x03, 0x9B, 0x87, 0x04, 0x0C, 0x37, 0x04, 0x8A, 0xA7, 0x05, 0x18, 0x84, 0x05, 0xB7, 0xB1,
  0x06, 0x6A, 0x4A, 0x07, 0x32, 0xAE, 0x08, 0x13, 0x85, 0x09, 0x0F, 0xCB, 0x0A, 0x2A, 0xDA, 0x0B, 0x68, 0x73,
  0x0C, 0xCC, 0xCC, 0x0E, 0x5C, 0xA1, 0x10, 0x1D, 0x3F, 0x12, 0x14, 0x9A, 0x14, 0x49, 0x60, 0x16, 0xC3, 0x10,
  0x19, 0x8A, 0x13, 0x1C, 0xA7, 0xD7, 0x20, 0x26, 0xF3, 0x24, 0x13, 0x46, 0x28, 0x7A, 0x26, 0x2D, 0x6A, 0x86,
  0x32, 0xF5, 0x2C, 0x39, 0x2C, 0xED, 0x40, 0x26, 0xE7, 0x47, 0xFA, 0xCC, 0x50, 0xC3, 0x35, 0x5A, 0x9D, 0xF7,
  0x65, 0xAC, 0x8C, 0x72, 0x14, 0x82, 0x80, 0x00, 0x00, 0x8F, 0x9E, 0x4C, 0xA1, 0x24, 0x77, 0xB4, 0xCE, 0x07,
  0xCA, 0xDD, 0xC7, 0xE3, 0x9E, 0xA8, 0xFF, 0x64, 0xC1
};

/*  The Volume Trim table converts the 20 or so volume trim values for each channel into logarithmically spaced
     values for the ADAU1701.  This is another tab in the Excel spreadsheet
*/
const byte Trim_Volume_table[]PROGMEM = {
  0x28, 0x7A, 0x26,
  0x2D, 0x6A, 0x86,
  0x32, 0xF5, 0x2C,
  0x39, 0x2C, 0xED,
  0x40, 0x26, 0xE7,
  0x47, 0xFA, 0xCC,
  0x50, 0xC3, 0x35,
  0x5A, 0x9D, 0xF7,
  0x65, 0xAC, 0x8C,
  0x72, 0x14, 0x82,
  0x80, 0x00, 0x00
};

const byte Delay_table[]PROGMEM = {
  0x1, 0x2, 0x3, 0x4, 0x5, 0x6,
  0x7, 0x9, 0xB, 0xE, 0x12, 0x16
};

/*  The original versions of this code "cleverly" converted the text entries in the HCI table values to floating point.  However, that approach
 *   requires some discipline to make sure the HCI table is in the right form, and it is probably "too cutsie" to be easily supported.  Instead,
 *   we have a look-up table that converts the HCI code into the corresponding floating point values.  This approach has its own issues, but it
 *   is somewhat easier to follow and it makes the library calls less dependent on the HCI. Remember:  HCI table has text; Lookup_tables.h has floating point
 */
 
const float EQ_Gain_table[] PROGMEM = {
  -10.0, -8.0, -6.0, -4.0, -2.0, 0.0, 2.0, 4.0, 6.0, 8.0, 10.0
};

const float TW_Xover_freq_table[] PROGMEM = {
  1200, 1600, 2000, 2400, 2800, 3200, 3600
};

const float WS_Xover_freq_table[] PROGMEM = {
  80, 100, 120, 150, 200
};

const float BSC_Freq_table[] PROGMEM = {
  300.0, 400.0, 500.0, 600.0, 700.0, 800.0, 900.0, 1000.0
};

const float BSC_Gain_table[] PROGMEM = {
  0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0
};

const float Bass_Freq_table[] PROGMEM = {
  15.0, 20.0, 30.0, 40.0, 50.0, 65.0, 80.0, 100.0, 120.0
};

const float Bass_Q_table[] PROGMEM = {
  0.5, 0.7, 1.0, 1.5, 2.2, 2.9
};

const float Bass_Gain_table[] PROGMEM = {
  1.0, 2.0, 3.0, 4.0, 5.0, 6.0
};

const byte SB_table_length = 36;
const byte SuperBass_freq_table[][SB_table_length]PROGMEM = {
  {0x00, 0x7F, 0xD5, 0x1B, 0xFF, 0x00, 0x55, 0xC9, 0x00, 0xFF, 0xAA, 0x1A, 0xFF, 0x80, 0x55, 0xAD, 0x00, 0x7F, 0xD5, 0x2A, 0x00, 0x7F, 0xAA, 0x53, 0x00, 0x00, 0x00, 0x0E, 0x00, 0x00, 0x00, 0x1D, 0x00, 0x00, 0x2A, 0xD6}, 	//20
  {0x00, 0x7F, 0xAA, 0x37, 0xFF, 0x00, 0xAB, 0x92, 0x00, 0xFF, 0x53, 0xFB, 0xFF, 0x80, 0xAB, 0x20, 0x00, 0x7F, 0xAA, 0x70, 0x00, 0x7F, 0x54, 0xE0, 0x00, 0x00, 0x00, 0x39, 0x00, 0x00, 0x00, 0x73, 0x00, 0x00, 0x55, 0x90},	//40
  {0x00, 0x7F, 0x7F, 0x52, 0xFF, 0x01, 0x01, 0x5B, 0x00, 0xFE, 0xFD, 0xA3, 0xFF, 0x81, 0x00, 0x5A, 0x00, 0x7F, 0x7F, 0xD3, 0x00, 0x7E, 0xFF, 0xA5, 0x00, 0x00, 0x00, 0x81, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x80, 0x2D},	//60
  {0x00, 0x7F, 0x54, 0x6E, 0xFF, 0x01, 0x57, 0x24, 0x00, 0xFE, 0xA7, 0x13, 0xFF, 0x81, 0x55, 0x5A, 0x00, 0x7F, 0x55, 0x52, 0x00, 0x7E, 0xAA, 0xA3, 0x00, 0x00, 0x00, 0xE5, 0x00, 0x00, 0x01, 0xCA, 0x00, 0x00, 0xAA, 0xAE},	//80
  {0x00, 0x7F, 0x29, 0x8A, 0xFF, 0x01, 0xAC, 0xEC, 0x00, 0xFE, 0x50, 0x4A, 0xFF, 0x81, 0xAA, 0x22, 0x00, 0x7F, 0x2A, 0xED, 0x00, 0x7E, 0x55, 0xDA, 0x00, 0x00, 0x01, 0x65, 0x00, 0x00, 0x02, 0xCA, 0x00, 0x00, 0xD5, 0x13},	//100
  {0x00, 0x7E, 0xBE, 0x52, 0xFF, 0x02, 0x83, 0x5C, 0x00, 0xFD, 0x76, 0x63, 0xFF, 0x82, 0x7D, 0x1A, 0x00, 0x7E, 0xC1, 0x6B, 0x00, 0x7D, 0x82, 0xD6, 0x00, 0x00, 0x03, 0x21, 0x00, 0x00, 0x06, 0x41, 0x00, 0x01, 0x3E, 0x95}, 	//150
  {0x00, 0x7E, 0x53, 0x1E, 0xFF, 0x03, 0x59, 0xC5, 0x00, 0xFC, 0x9B, 0x26, 0xFF, 0x83, 0x4E, 0xAF, 0x00, 0x7E, 0x58, 0x96, 0x00, 0x7C, 0xB1, 0x2C, 0x00, 0x00, 0x05, 0x8B, 0x00, 0x00, 0x0B, 0x15, 0x00, 0x01, 0xA7, 0x6A},	//200
  {0x00, 0x7D, 0xE7, 0xEE, 0xFF, 0x04, 0x30, 0x25, 0x00, 0xFB, 0xBE, 0x98, 0xFF, 0x84, 0x1E, 0xE2, 0x00, 0x7D, 0xF0, 0x6C, 0x00, 0x7B, 0xE0, 0xD7, 0x00, 0x00, 0x08, 0xA2, 0x00, 0x00, 0x11, 0x43, 0x00, 0x02, 0x0F, 0x94},	//250
  {0x00, 0x7D, 0x7C, 0xC3, 0xFF, 0x05, 0x06, 0x7A, 0x00, 0xFA, 0xE0, 0xC0, 0xFF, 0x84, 0xED, 0xB3, 0x00, 0x7D, 0x88, 0xE9, 0x00, 0x7B, 0x11, 0xD3, 0x00, 0x00, 0x0C, 0x63, 0x00, 0x00, 0x18, 0xC7, 0x00, 0x02, 0x77, 0x17}	//300
};

const byte SuperBass_Int_and_Gain_table[][4]PROGMEM = {
  {0x00, 0x0C, 0xCC, 0xCD}, {0x00, 0x19, 0x99, 0x9A}, {0x00, 0x26, 0x66, 0x66}, {0x00, 0x33, 0x33, 0x33}, {0x00, 0x40, 0x00, 0x00}, {0x00, 0x4C, 0xCC, 0xCD}, {0x00, 0x59, 0x99, 0x9A}, {0x00, 0x66, 0x66, 0x66}, {0x00, 0x73, 0x33, 0x33}, {0x00, 0x80, 0x00, 0x00},
  {0x00, 0x8C, 0xCC, 0xCD}, {0x00, 0x99, 0x99, 0x9A}, {0x00, 0xA6, 0x66, 0x66}, {0x00, 0xB3, 0x33, 0x33}, {0x00, 0xC0, 0x00, 0x00}, {0x00, 0xCC, 0xCC, 0xCD}, {0x00, 0xD9, 0x99, 0x9A}, {0x00, 0xE6, 0x66, 0x66}, {0x00, 0xF3, 0x33, 0x33}, {0x01, 0x00, 0x00, 0x00},
  {0x01, 0x0C, 0xCC, 0xCD}, {0x01, 0x19, 0x99, 0x9A}, {0x01, 0x26, 0x66, 0x66}, {0x01, 0x33, 0x33, 0x33}, {0x01, 0x40, 0x00, 0x00}, {0x01, 0x4C, 0xCC, 0xCD}, {0x01, 0x59, 0x99, 0x9A}, {0x01, 0x66, 0x66, 0x66}, {0x01, 0x73, 0x33, 0x33}, {0x01, 0x80, 0x00, 0x00}
};

const byte DB_Threshold_table[][4]PROGMEM = {
  {0x00, 0xFF, 0x64, 0xC1}, {0x00, 0xEA, 0x44, 0x32}, {0x00, 0xD6, 0xE3, 0x0D}, {0x00, 0xC5, 0x1C, 0x50}, {0x00, 0xB4, 0xCE, 0x08}, {0x00, 0xA5, 0xD9, 0x10}, {0x00, 0x98, 0x20, 0xD7},    //-24, -21. -18, -15, -12, -9, -6 
  {0x00, 0x8B, 0x8B, 0x2A}, {0x00, 0x80, 0x00, 0x00}, {0x00, 0x6E, 0xD7, 0xEB}, {0x00, 0x5F, 0xFC, 0x89}, {0x00, 0x53, 0x1E, 0xF0}, {0x00, 0x47, 0xFA, 0xCD}     //-3, -0. +5, +10, +15, +20 
};

const byte DB_Time_constant_table[][4]PROGMEM = {
  {0x00, 0x80, 0x00, 0x00}, {0x00, 0x00, 0x0D, 0x8F}, {0x00, 0x00, 0x06, 0xC8}, {0x00, 0x00, 0x04, 0x85}, {0x00, 0x00, 0x03, 0x64},     //0, 50. 100, 150, 200 
  {0x00, 0x00, 0x02, 0xB6}, {0x00, 0x00, 0x02, 0x43}, {0x00, 0x00, 0x01, 0xB2}, {0x00, 0x00, 0x01, 0x5B}    //250, 300, 400, 500 
};

const byte DB_Bass_Boost_table[][4]PROGMEM = {
  {0x00, 0x80, 0x00, 0x00}, {0x00, 0x56, 0x89, 0xE3}, {0x00, 0x3D, 0x43, 0xC0}, {0x00, 0x2C, 0x61, 0xE0},      	//0, 3. 6, 9
  {0x00, 0x1E, 0xB4, 0x7F}, {0x00, 0x15, 0xBC, 0xCE}, {0x00, 0x10, 0x1D, 0x3F}, {0x00, 0x0C, 0xCC, 0xCD}    	//12, 15, 18, 20 
};


const byte DB_Bass_Freq_table[][4]PROGMEM = {
  {0x00, 0x00, 0x55, 0xC9}, {0x00, 0x00, 0xAB, 0x93}, {0x00, 0x01, 0x01, 0x5C}, {0x00, 0x01, 0x57, 0x25}, {0x00, 0x01, 0xAC, 0xEE},     //20, 40, 60, 80, 100 
  {0x00, 0x02, 0x83, 0x63}, {0x00, 0x03, 0x59, 0xD7}, {0x00, 0x04, 0x30, 0x48}, {0x00, 0x05, 0x06, 0xB7}    //150, 200, 250, 300 
};

const byte DB_Bass_Coef_table[][4]PROGMEM = {
  {0x00, 0x00, 0x00, 0x39}, {0x00, 0x00, 0x00, 0x73}, {0x00, 0x00, 0x00, 0x39}, {0x00, 0xFF, 0x0D, 0x53}, {0xFF, 0x80, 0xF1, 0xC8},	//20Hz
  {0x00, 0x00, 0x00, 0xE4}, {0x00, 0x00, 0x01, 0xC9}, {0x00, 0x00, 0x00, 0xE4}, {0x00, 0xFE, 0x1A, 0xA8}, {0xFF, 0x81, 0xE1, 0xC7},	//40Hz
  {0x00, 0x00, 0x02, 0x00}, {0x00, 0x00, 0x04, 0x00}, {0x00, 0x00, 0x02, 0x00}, {0x00, 0xFD, 0x28, 0x00}, {0xFF, 0x82, 0xD0, 0x01},	//60Hz
  {0x00, 0x00, 0x03, 0x8A}, {0x00, 0x00, 0x07, 0x15}, {0x00, 0x00, 0x03, 0x8A}, {0x00, 0xFC, 0x35, 0x5D}, {0xFF, 0x83, 0xBC, 0x79},	//80Hz
  {0x00, 0x00, 0x05, 0x83}, {0x00, 0x00, 0x0B, 0x06}, {0x00, 0x00, 0x05, 0x83}, {0x00, 0xFB, 0x42, 0xC1}, {0xFF, 0x84, 0xA7, 0x32},	//100Hz
  {0x00, 0x00, 0x0C, 0x4A}, {0x00, 0x00, 0x18, 0x95}, {0x00, 0x00, 0x0C, 0x4A}, {0x00, 0xF8, 0xE4, 0x62}, {0xFF, 0x86, 0xEA, 0x75},	//150Hz
  {0x00, 0x00, 0x15, 0xA7}, {0x00, 0x00, 0x2B, 0x4E}, {0x00, 0x00, 0x15, 0xA7}, {0x00, 0xF6, 0x86, 0x4D}, {0xFF, 0x89, 0x23, 0x17},	//200Hz
  {0x00, 0x00, 0x21, 0x87}, {0x00, 0x00, 0x43, 0x0F}, {0x00, 0x00, 0x21, 0x87}, {0x00, 0xF4, 0x28, 0x99}, {0xFF, 0x8B, 0x51, 0x4A},	//250Hz
  {0x00, 0x00, 0x2F, 0xDA}, {0x00, 0x00, 0x5F, 0xB4}, {0x00, 0x00, 0x2F, 0xDA}, {0x00, 0xF1, 0xCB, 0x58}, {0xFF, 0x8D, 0x75, 0x40}	//300Hz
};

